'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.detectIsMetatagsBox = exports.MetatagsBox = exports.Metatags = void 0;
const core_1 = require('@dark-engine/core');
const constants_1 = require('../constants');
const utils_1 = require('../utils');
const $$metatags = Symbol('metatags');
const Metatags = (0, core_1.component)(
  ({ slot }) => {
    const { isServer, isHydration } = (0, core_1.__useSSR)();
    const scope = (0, core_1.useMemo)(() => ({ isDirty: false }), []);
    const vNodes = createVNodes(slot);
    if (isServer && !scope.isDirty) {
      const $scope = (0, core_1.$$scope)();
      const emitter = $scope.getEmitter();
      const off = emitter.on('chunk', fiber => {
        if (detectIsMetatags(fiber.inst)) {
          emitter.emit('box', new MetatagsBox(vNodes));
          off();
        }
      });
      scope.isDirty = true;
    }
    (0, core_1.useLayoutEffect)(() => {
      if (isHydration) return;
      for (const vNode of vNodes) {
        const { children, attrs } = vNode;
        const element = resolveElement(vNode);
        if (!element) continue;
        const text = children.map(x => ((0, core_1.detectIsTextVirtualNode)(x) ? x.value : x)).join('');
        setAttributes(element, attrs);
        setTextContent(element, text);
      }
    });
    return null;
  },
  { token: $$metatags, displayName: 'Metatags' },
);
exports.Metatags = Metatags;
function createVNodes(slot) {
  const vNodes = ((0, core_1.detectIsArray)(slot) ? slot : [slot]).map(x => {
    if ((0, core_1.detectIsVirtualNodeFactory)(x)) {
      return x();
    } else {
      (0, utils_1.illegal)(`Metatags supports only tags!`);
    }
  });
  return vNodes;
}
function resolveElement(vNode) {
  if (vNode.name === constants_1.TITLE_TAG || vNode.name === constants_1.META_TAG) {
    return document.querySelector(`${constants_1.HEAD_TAG} ${createSelector(vNode)}`);
  }
  return null;
}
function createSelector(vNode) {
  const attrs =
    vNode.name === constants_1.META_TAG
      ? createAttributeSelector(constants_1.NAME_ATTR, vNode.attrs[constants_1.NAME_ATTR])
      : '';
  const selector = `${vNode.name}${attrs}`;
  return selector;
}
const createAttributeSelector = (name, value) => (!(0, core_1.detectIsEmpty)(value) ? `[${name}="${value}"]` : '');
function setAttributes(element, attrs) {
  for (const key in attrs) {
    if (key === constants_1.DANGER_HTML_ATTR) continue;
    const value = String(attrs[key]);
    element.getAttribute(key) !== value && element.setAttribute(key, value);
  }
}
const setTextContent = (element, text) => element.textContent !== text && (element.textContent = text);
class MetatagsBox {
  vNodes;
  constructor(vNodes) {
    this.vNodes = vNodes;
  }
}
exports.MetatagsBox = MetatagsBox;
const detectIsMetatags = x => (0, core_1.detectIsComponent)(x) && x.token === $$metatags;
const detectIsMetatagsBox = x => x instanceof MetatagsBox;
exports.detectIsMetatagsBox = detectIsMetatagsBox;
//# sourceMappingURL=metatags.js.map
